@@ -387,7 +387,7 @@ defimpl Inspect, for: Float do
       if abs >= 1.0 and abs < 1.0e16 and trunc(float) == float do
         [Integer.to_string(trunc(float)), ?., ?0]
       else
-        :io_lib_format.fwrite_g(float)
+        :erlang.float_to_list(float, [:short])
       end
 
     color(IO.iodata_to_binary(formatted), :number, opts)
@@ -396,24 +396,38 @@ end
 
 defimpl Inspect, for: Regex do
   def inspect(regex = %{opts: regex_opts}, opts) when is_list(regex_opts) do
-    concat([
-      "Regex.compile!(",
-      Inspect.BitString.inspect(regex.source, opts),
-      ", ",
-      Inspect.List.inspect(regex_opts, opts),
-      ")"
-    ])
+    case translate_options(regex_opts, []) do
+      :error ->
+        concat([
+          "Regex.compile!(",
+          Inspect.BitString.inspect(regex.source, opts),
+          ", ",
+          Inspect.List.inspect(regex_opts, opts),
+          ")"
+        ])
+
+      translated_opts ->
+        {escaped, _} =
+          regex.source
+          |> normalize(<<>>)
+          |> Identifier.escape(?/, :infinity, &escape_map/1)
+
+        source = IO.iodata_to_binary([?~, ?r, ?/, escaped, ?/, translated_opts])
+        color(source, :regex, opts)
+    end
   end
 
-  def inspect(regex, opts) do
-    {escaped, _} =
-      regex.source
-      |> normalize(<<>>)
-      |> Identifier.escape(?/, :infinity, &escape_map/1)
+  defp translate_options([:dotall, {:newline, :anycrlf} | t], acc),
+    do: translate_options(t, [?s | acc])
 
-    source = IO.iodata_to_binary([?~, ?r, ?/, escaped, ?/, regex.opts])
-    color(source, :regex, opts)
-  end
+  defp translate_options([:unicode, :ucp | t], acc), do: translate_options(t, [?u | acc])
+  defp translate_options([:caseless | t], acc), do: translate_options(t, [?i | acc])
+  defp translate_options([:extended | t], acc), do: translate_options(t, [?x | acc])
+  defp translate_options([:firstline | t], acc), do: translate_options(t, [?f | acc])
+  defp translate_options([:ungreedy | t], acc), do: translate_options(t, [?U | acc])
+  defp translate_options([:multiline | t], acc), do: translate_options(t, [?m | acc])
+  defp translate_options([], acc), do: acc
+  defp translate_options(_t, _acc), do: :error
 
   defp normalize(<<?\\, ?\\, rest::binary>>, acc), do: normalize(rest, <<acc::binary, ?\\, ?\\>>)
   defp normalize(<<?\\, ?/, rest::binary>>, acc), do: normalize(rest, <<acc::binary, ?/>>)
@@ -449,7 +463,7 @@ defimpl Inspect, for: Function do
 
       match?(@elixir_compiler ++ _, Atom.to_charlist(mod)) ->
         if function_exported?(mod, :__RELATIVE__, 0) do
-          "#Function<#{uniq(fun_info)} in file:#{mod.__RELATIVE__}>"
+          "#Function<#{uniq(fun_info)} in file:#{mod.__RELATIVE__()}>"
         else
           default_inspect(mod, fun_info)
         end
