@@ -161,7 +161,7 @@ defmodule Integer do
   @doc since: "1.4.0"
   @spec floor_div(integer, neg_integer | pos_integer) :: integer
   def floor_div(dividend, divisor) do
-    if dividend * divisor < 0 and rem(dividend, divisor) != 0 do
+    if :erlang.xor(dividend < 0, divisor < 0) and rem(dividend, divisor) != 0 do
       div(dividend, divisor) - 1
     else
       div(dividend, divisor)
@@ -362,25 +362,25 @@ defmodule Integer do
   ## Examples
 
       iex> Integer.to_charlist(123)
-      '123'
+      ~c"123"
 
       iex> Integer.to_charlist(+456)
-      '456'
+      ~c"456"
 
       iex> Integer.to_charlist(-789)
-      '-789'
+      ~c"-789"
 
       iex> Integer.to_charlist(0123)
-      '123'
+      ~c"123"
 
       iex> Integer.to_charlist(100, 16)
-      '64'
+      ~c"64"
 
       iex> Integer.to_charlist(-100, 16)
-      '-64'
+      ~c"-64"
 
       iex> Integer.to_charlist(882_681_651, 36)
-      'ELIXIR'
+      ~c"ELIXIR"
 
   """
   @spec to_charlist(integer, 2..36) :: charlist
