@@ -176,10 +176,15 @@ defmodule Module.Types.Of do
   # TODO: Use the struct default values to define the default types.
   def struct(struct, args_types, default_handling, meta, stack, context) do
     context = remote(struct, :__struct__, 0, meta, stack, context)
+
+    info =
+      struct.__info__(:struct) ||
+        raise "expected #{inspect(struct)} to return struct metadata, but got none"
+
     term = term()
 
     defaults =
-      for %{field: field} <- struct.__info__(:struct), field != :__struct__ do
+      for %{field: field} <- info, field != :__struct__ do
         {field, term}
       end
 
@@ -700,14 +705,16 @@ defmodule Module.Types.Of do
           " is undefined (module ",
           inspect(module),
           " is not available or is yet to be defined)"
-        ])
+        ]),
+      group: true
     }
   end
 
   def format_diagnostic({:undefined_function, module, :__struct__, 0, _exports}) do
     %{
       message:
-        "struct #{inspect(module)} is undefined (there is such module but it does not define a struct)"
+        "struct #{inspect(module)} is undefined (there is such module but it does not define a struct)",
+      group: true
     }
   end
 
@@ -718,7 +725,8 @@ defmodule Module.Types.Of do
           Exception.format_mfa(module, fun, arity),
           " is undefined or private",
           UndefinedFunctionError.hint_for_loaded_module(module, fun, arity, exports)
-        ])
+        ]),
+      group: true
     }
   end
 
@@ -729,7 +737,8 @@ defmodule Module.Types.Of do
           Exception.format_mfa(module, fun, arity),
           " is deprecated. ",
           reason
-        ])
+        ]),
+      group: true
     }
   end
 
@@ -741,7 +750,8 @@ defmodule Module.Types.Of do
           inspect(module),
           " before invoking the macro ",
           Exception.format_mfa(module, fun, arity)
-        ])
+        ]),
+      group: true
     }
   end
 
