@@ -64,6 +64,9 @@ defmodule Path do
       iex> Path.absname("../x", "bar")
       "bar/../x"
 
+      iex> Path.absname("foo", fn -> "lazy" end)
+      "lazy/foo"
+
   """
   @spec absname(t, t | (-> t)) :: binary
   def absname(path, relative_to) do
@@ -169,11 +172,17 @@ defmodule Path do
   Converts the path to an absolute one, expanding
   any `.` and `..` components and a leading `~`.
 
+  If a relative path is provided it is expanded relatively to
+  the current working directory.
+
   ## Examples
 
       Path.expand("/foo/bar/../baz")
       #=> "/foo/baz"
 
+      Path.expand("foo/bar/../baz")
+      #=> "$PWD/foo/baz"
+
   """
   @spec expand(t) :: binary
   def expand(path) do
@@ -258,7 +267,7 @@ defmodule Path do
       Path.relative("/bar/foo.ex")      #=> "bar/foo.ex"
 
   """
-  # Note this function does not expand paths because the behaviour
+  # Note this function does not expand paths because the behavior
   # is ambiguous. If we expand it before converting to relative, then
   # "/usr/../../foo" means "/foo". If we expand it after, it means "../foo".
   # We could expand only relative paths but it is best to say it never
