@@ -135,14 +135,14 @@ defmodule Task do
 
   With `Task.Supervisor`, it is easy to dynamically start tasks across nodes:
 
-      # On the remote node named :remote@local
+      # First on the remote node named :remote@local
       Task.Supervisor.start_link(name: MyApp.DistSupervisor)
 
-      # On the client
+      # Then on the local client node
       supervisor = {MyApp.DistSupervisor, :remote@local}
       Task.Supervisor.async(supervisor, MyMod, :my_fun, [arg1, arg2, arg3])
 
-  Note that, when working with distributed tasks, one should use the
+  Note that, as above, when working with distributed tasks, one should use the
   `Task.Supervisor.async/5` function that expects explicit module, function,
   and arguments, instead of `Task.Supervisor.async/3` that works with anonymous
   functions. That's because anonymous functions expect the same module version
@@ -293,6 +293,17 @@ defmodule Task do
   """
   @opaque ref :: reference()
 
+  @typedoc """
+  Options given to `async_stream` functions.
+  """
+  @typedoc since: "1.17.0"
+  @type async_stream_option ::
+          {:max_concurrency, pos_integer()}
+          | {:ordered, boolean()}
+          | {:timeout, timeout()}
+          | {:on_timeout, :exit | :kill_task}
+          | {:zip_input_on_exit, boolean()}
+
   defguardp is_timeout(timeout)
             when timeout == :infinity or (is_integer(timeout) and timeout >= 0)
 
@@ -681,7 +692,8 @@ defmodule Task do
   example above.
   """
   @doc since: "1.4.0"
-  @spec async_stream(Enumerable.t(), module, atom, [term], keyword) :: Enumerable.t()
+  @spec async_stream(Enumerable.t(), module, atom, [term], [async_stream_option]) ::
+          Enumerable.t()
   def async_stream(enumerable, module, function_name, args, options \\ [])
       when is_atom(module) and is_atom(function_name) and is_list(args) do
     build_stream(enumerable, {module, function_name, args}, options)
@@ -710,13 +722,15 @@ defmodule Task do
   See `async_stream/5` for discussion, options, and more examples.
   """
   @doc since: "1.4.0"
-  @spec async_stream(Enumerable.t(), (term -> term), keyword) :: Enumerable.t()
+  @spec async_stream(Enumerable.t(), (term -> term), [async_stream_option]) :: Enumerable.t()
   def async_stream(enumerable, fun, options \\ [])
       when is_function(fun, 1) and is_list(options) do
     build_stream(enumerable, fun, options)
   end
 
   defp build_stream(enumerable, fun, options) do
+    options = Task.Supervised.validate_stream_options(options)
+
     fn acc, acc_fun ->
       owner = get_owner(self())
 
@@ -1181,7 +1195,7 @@ defmodule Task do
   given time. All other tasks will have been shut down using
   the `Task.shutdown/2` call.
 
-  As a convenience, you can achieve a similar behaviour to above
+  As a convenience, you can achieve a similar behavior to above
   by specifying the `:on_timeout` option to be `:kill_task` (or
   `:ignore`). See `Task.await_many/2` if you would rather exit
   the caller process on timeout.
