@@ -2,12 +2,13 @@
 %% private to the Elixir compiler and reserved to be used by Elixir only.
 -module(elixir).
 -behaviour(application).
--export([start_cli/0, start/0, start_iex/0]).
+-export([start_cli/0, start/0]).
 -export([start/2, stop/1, config_change/3]).
 -export([
   string_to_tokens/5, tokens_to_quoted/3, 'string_to_quoted!'/5,
   env_for_eval/1, quoted_to_erl/2, eval_forms/3, eval_quoted/3,
-  eval_quoted/4, eval_local_handler/2, eval_external_handler/3
+  eval_quoted/4, eval_local_handler/2, eval_external_handler/3,
+  format_token_error/1
 ]).
 -include("elixir.hrl").
 -define(system, 'Elixir.System').
@@ -139,10 +140,10 @@ preload_common_modules() ->
 parse_otp_release() ->
   %% Whenever we change this check, we should also change Makefile.
   case string:to_integer(erlang:system_info(otp_release)) of
-    {Num, _} when Num >= 24 ->
+    {Num, _} when Num >= 25 ->
       Num;
     _ ->
-      io:format(standard_error, "ERROR! Unsupported Erlang/OTP version, expected Erlang/OTP 24+~n", []),
+      io:format(standard_error, "ERROR! Unsupported Erlang/OTP version, expected Erlang/OTP 25+~n", []),
       erlang:halt(1)
   end.
 
@@ -176,16 +177,12 @@ check_file_encoding(Encoding) ->
   end.
 
 %% Boot and process given options. Invoked by Elixir's script.
-
-%% TODO: Delete prim_tty branches and -user on Erlang/OTP 26.
-%% TODO: Remove IEx.CLI module
-%% TODO: Replace "-user elixir" and "-s elixir start_$MODE"
-%% by calls to "-s elixir cli" and "-e iex:cli()"
+%% TODO: Delete prim_tty branches on Erlang/OTP 26.
 
 start() ->
   case code:ensure_loaded(prim_tty) of
     {module, _} ->
-      user_drv:start(#{initial_shell => noshell});
+      user_drv:start(#{initial_shell => iex:shell()});
     {error, _} ->
       case init:get_argument(elixir_root) of
         {ok, [[Root]]} -> code:add_patha(Root ++ "/iex/ebin");
@@ -207,18 +204,7 @@ start_cli() ->
     {error, _}  -> ok
   end,
 
-  'Elixir.Kernel.CLI':main(init:get_plain_arguments()),
-  elixir_config:booted().
-
-start_iex() ->
-  case code:ensure_loaded(prim_tty) of
-    {module, _} ->
-      start_cli(),
-      iex:cli();
-
-    {error, _} ->
-      ok
-  end.
+  'Elixir.Kernel.CLI':main(init:get_plain_arguments()).
 
 %% EVAL HOOKS
 
@@ -361,7 +347,7 @@ eval_forms(Tree, Binding, OrigE, Opts) ->
 
       %% We use remote names so eval works across Elixir versions.
       LocalHandler = {value, fun ?MODULE:eval_local_handler/2},
-      ExternalHandler = eval_external_handler(),
+      ExternalHandler = {value, fun ?MODULE:eval_external_handler/3},
 
       {value, Value, NewBinding} =
         try
@@ -394,11 +380,6 @@ eval_local_handler(FunName, Args) ->
   Exception = 'Elixir.UndefinedFunctionError':exception(Opts),
   erlang:raise(error, Exception, Stack).
 
-%% TODO: Remove conditional once we require Erlang/OTP 25+.
--if(?OTP_RELEASE >= 25).
-eval_external_handler() ->
-  {value, fun ?MODULE:eval_external_handler/3}.
-
 eval_external_handler(Ann, FunOrModFun, Args) ->
   try
     case FunOrModFun of
@@ -461,10 +442,6 @@ drop_common([_ | T1], T2, ToDrop) -> drop_common(T1, T2, ToDrop);
 drop_common([], [{?MODULE, _, _, _} | T2], _ToDrop) -> T2;
 drop_common([], [_ | T2], true) -> T2;
 drop_common([], T2, _) -> T2.
--else.
-eval_external_handler() -> none.
-eval_external_handler(_Ann, _FunOrModFun, _Args) -> error(unused).
--endif.
 
 %% Converts a quoted expression to Erlang abstract format
 
@@ -483,18 +460,21 @@ quoted_to_erl(Quoted, ErlS, ExS, Env) ->
 
 string_to_tokens(String, StartLine, StartColumn, File, Opts) when is_integer(StartLine), is_binary(File) ->
   case elixir_tokenizer:tokenize(String, StartLine, StartColumn, Opts) of
-    {ok, _Line, _Column, [], Tokens} ->
-      {ok, Tokens};
-    {ok, _Line, _Column, Warnings, Tokens} ->
+    {ok, _Line, _Column, [], Tokens, Terminators} ->
+      {ok, lists:reverse(Tokens, Terminators)};
+    {ok, _Line, _Column, Warnings, Tokens, Terminators} ->
       (lists:keyfind(emit_warnings, 1, Opts) /= {emit_warnings, false}) andalso
         [elixir_errors:erl_warn(L, File, M) || {L, M} <- lists:reverse(Warnings)],
-      {ok, Tokens};
-    {error, {Location, {ErrorPrefix, ErrorSuffix}, Token}, _Rest, _Warnings, _SoFar} ->
-      {error, {Location, {to_binary(ErrorPrefix), to_binary(ErrorSuffix)}, to_binary(Token)}};
-    {error, {Location, Error, Token}, _Rest, _Warnings, _SoFar} ->
-      {error, {Location, to_binary(Error), to_binary(Token)}}
+      {ok, lists:reverse(Tokens, Terminators)};
+    {error, Info, _Rest, _Warnings, _SoFar} ->
+      {error, format_token_error(Info)}
   end.
 
+format_token_error({Location, {ErrorPrefix, ErrorSuffix}, Token}) ->
+  {Location, {to_binary(ErrorPrefix), to_binary(ErrorSuffix)}, to_binary(Token)};
+format_token_error({Location, Error, Token}) ->
+  {Location, to_binary(Error), to_binary(Token)}.
+
 tokens_to_quoted(Tokens, WarningFile, Opts) ->
   handle_parsing_opts(WarningFile, Opts),
 
