@@ -181,25 +181,23 @@ defmodule CodeTest do
       end
     end
 
-    if System.otp_release() >= "25" do
-      test "includes eval file in stacktrace" do
-        try do
-          Code.eval_string("<<a::size(b)>>", [a: :a, b: :b], file: "myfile")
-        rescue
-          _ ->
-            assert Exception.format_stacktrace(__STACKTRACE__) =~ "myfile:1"
-        end
+    test "includes eval file in stacktrace" do
+      try do
+        Code.eval_string("<<a::size(b)>>", [a: :a, b: :b], file: "myfile")
+      rescue
+        _ ->
+          assert Exception.format_stacktrace(__STACKTRACE__) =~ "myfile:1"
+      end
 
-        try do
-          Code.eval_string(
-            "Enum.map([a: :a, b: :b], fn {a, b} -> <<a::size(b)>> end)",
-            [],
-            file: "myfile"
-          )
-        rescue
-          _ ->
-            assert Exception.format_stacktrace(__STACKTRACE__) =~ "myfile:1"
-        end
+      try do
+        Code.eval_string(
+          "Enum.map([a: :a, b: :b], fn {a, b} -> <<a::size(b)>> end)",
+          [],
+          file: "myfile"
+        )
+      rescue
+        _ ->
+          assert Exception.format_stacktrace(__STACKTRACE__) =~ "myfile:1"
       end
     end
 
@@ -213,30 +211,6 @@ defmodule CodeTest do
              end) =~ "an __ENV__ with outdated compilation information was given to eval"
     end
 
-    test "emits checker warnings" do
-      output =
-        ExUnit.CaptureIO.capture_io(:stderr, fn ->
-          Code.eval_string(File.read!(fixture_path("checker_warning.exs")), [])
-        end)
-
-      assert output =~ "incompatible types"
-    after
-      :code.purge(CodeTest.CheckerWarning)
-      :code.delete(CodeTest.CheckerWarning)
-    end
-
-    test "captures checker diagnostics" do
-      {{{:module, _, _, _}, _}, diagnostics} =
-        Code.with_diagnostics(fn ->
-          Code.eval_string(File.read!(fixture_path("checker_warning.exs")), [])
-        end)
-
-      assert [%{message: "incompatible types:" <> _}] = diagnostics
-    after
-      :code.purge(CodeTest.CheckerWarning)
-      :code.delete(CodeTest.CheckerWarning)
-    end
-
     test "formats diagnostic file paths as relatives" do
       {_, diagnostics} =
         Code.with_diagnostics(fn ->
@@ -301,7 +275,7 @@ defmodule CodeTest do
 
       quoted = quote(do: alias(:dict, as: MyDict))
       {:dict, [], env} = Code.eval_quoted_with_env(quoted, [], env)
-      assert Macro.Env.fetch_alias(env, :MyDict) == {:ok, :dict}
+      assert Keyword.fetch(env.aliases, Elixir.MyDict) == {:ok, :dict}
     end
 
     test "manages env vars" do
@@ -339,19 +313,17 @@ defmodule CodeTest do
                {[{{:x, :foo}, 2}], [x: :foo]}
     end
 
-    if :erlang.system_info(:otp_release) >= ~c"25" do
-      test "undefined function" do
-        env = Code.env_for_eval(__ENV__)
-        quoted = quote do: foo()
+    test "undefined function" do
+      env = Code.env_for_eval(__ENV__)
+      quoted = quote do: foo()
 
-        assert_exception(
-          UndefinedFunctionError,
-          ["** (UndefinedFunctionError) function foo/0 is undefined (there is no such import)"],
-          fn ->
-            Code.eval_quoted_with_env(quoted, [], env)
-          end
-        )
-      end
+      assert_exception(
+        UndefinedFunctionError,
+        ["** (UndefinedFunctionError) function foo/0 is undefined (there is no such import)"],
+        fn ->
+          Code.eval_quoted_with_env(quoted, [], env)
+        end
+      )
     end
 
     defmodule Tracer do
@@ -402,47 +374,6 @@ defmodule CodeTest do
       assert Code.compile_file(fixture_path("code_sample.exs")) == []
       refute fixture_path("code_sample.exs") in Code.required_files()
     end
-
-    test "emits checker warnings" do
-      output =
-        ExUnit.CaptureIO.capture_io(:stderr, fn ->
-          Code.compile_file(fixture_path("checker_warning.exs"))
-        end)
-
-      assert output =~ "incompatible types"
-    after
-      :code.purge(CodeTest.CheckerWarning)
-      :code.delete(CodeTest.CheckerWarning)
-    end
-
-    test "captures checker diagnostics" do
-      {[{CodeTest.CheckerWarning, _}], diagnostics} =
-        Code.with_diagnostics(fn ->
-          Code.compile_file(fixture_path("checker_warning.exs"))
-        end)
-
-      assert [%{message: "incompatible types:" <> _}] = diagnostics
-    after
-      :code.purge(CodeTest.CheckerWarning)
-      :code.delete(CodeTest.CheckerWarning)
-    end
-
-    test "captures checker diagnostics with logging" do
-      output =
-        ExUnit.CaptureIO.capture_io(:stderr, fn ->
-          {[{CodeTest.CheckerWarning, _}], diagnostics} =
-            Code.with_diagnostics([log: true], fn ->
-              Code.compile_file(fixture_path("checker_warning.exs"))
-            end)
-
-          assert [%{message: "incompatible types:" <> _}] = diagnostics
-        end)
-
-      assert output =~ "incompatible types"
-    after
-      :code.purge(CodeTest.CheckerWarning)
-      :code.delete(CodeTest.CheckerWarning)
-    end
   end
 
   test "require_file/1" do
@@ -526,18 +457,6 @@ defmodule CodeTest do
       :code.delete(CompileSimpleSample)
     end
 
-    test "emits checker warnings" do
-      output =
-        ExUnit.CaptureIO.capture_io(:stderr, fn ->
-          Code.compile_string(File.read!(fixture_path("checker_warning.exs")))
-        end)
-
-      assert output =~ "incompatible types"
-    after
-      :code.purge(CodeTest.CheckerWarning)
-      :code.delete(CodeTest.CheckerWarning)
-    end
-
     test "works across lexical scopes" do
       assert [{CompileCrossSample, _}] =
                Code.compile_string("CodeTest.genmodule CompileCrossSample")
@@ -653,7 +572,7 @@ defmodule Code.SyncTest do
 
   import PathHelpers
 
-  if :erlang.system_info(:otp_release) >= ~c"26" do
+  if System.otp_release() >= "26" do
     defp assert_cached(path) do
       assert find_path(path) != :nocache
     end
@@ -728,7 +647,7 @@ defmodule Code.SyncTest do
   end
 
   test "purges compiler modules" do
-    quoted = quote(do: Agent.start_link(fn -> :ok end))
+    quoted = quote(do: :ok)
     Code.compile_quoted(quoted)
 
     {:ok, claimed} = Code.purge_compiler_modules()
@@ -736,6 +655,12 @@ defmodule Code.SyncTest do
 
     {:ok, claimed} = Code.purge_compiler_modules()
     assert claimed == 0
+
+    quoted = quote(do: Agent.start_link(fn -> :ok end))
+    Code.compile_quoted(quoted)
+
+    {:ok, claimed} = Code.purge_compiler_modules()
+    assert claimed == 0
   end
 
   test "returns previous options when setting compiler options" do
