@@ -534,24 +534,15 @@ defmodule ExceptionTest do
       assert blame_message([], & &1.foo()) ==
                "you attempted to apply a function named :foo on []. If you are using Kernel.apply/3, make sure " <>
                  "the module is an atom. If you are using the dot syntax, such as " <>
-                 "module.function(), make sure the left-hand side of the dot is a module atom"
+                 "module.function(), make sure the left-hand side of the dot is an atom representing a module"
 
       assert blame_message([], &apply(&1, :foo, [])) ==
                "you attempted to apply a function named :foo on []. If you are using Kernel.apply/3, make sure " <>
                  "the module is an atom. If you are using the dot syntax, such as " <>
-                 "module.function(), make sure the left-hand side of the dot is a module atom"
+                 "module.function(), make sure the left-hand side of the dot is an atom representing a module"
 
-      assert blame_message([], &apply(Kernel, &1, [1, 2])) ==
-               "you attempted to apply a function named [] on module Kernel. However, [] is not a valid function name. " <>
-                 "Function names (the second argument of apply) must always be an atom"
-
-      assert blame_message(123, &apply(Kernel, :+, &1)) ==
-               "you attempted to apply a function named :+ on module Kernel with arguments 123. " <>
-                 "Arguments (the third argument of apply) must always be a proper list"
-
-      assert blame_message(123, &apply(Kernel, :+, [&1 | 456])) ==
-               "you attempted to apply a function named :+ on module Kernel with arguments [123 | 456]. " <>
-                 "Arguments (the third argument of apply) must always be a proper list"
+      assert blame_message([], &apply(&1, :foo, [1, 2])) ==
+               "you attempted to apply a function on []. Modules (the first argument of apply) must always be an atom"
     end
 
     test "annotates function clause errors" do
@@ -687,11 +678,6 @@ defmodule ExceptionTest do
       assert blame_message(nil, & &1.foo) ==
                "key :foo not found in: nil\n\nIf you are using the dot syntax, " <>
                  "such as map.field, make sure the left-hand side of the dot is a map"
-
-      # we use `Code.eval_string/1` to escape the formatter and warnings
-      assert blame_message("nil.foo", &Code.eval_string/1) ==
-               "key :foo not found in: nil\n\nIf you are using the dot syntax, " <>
-                 "such as map.field, make sure the left-hand side of the dot is a map"
     end
 
     test "annotates undefined function clause error with nil hints" do
@@ -1001,17 +987,15 @@ defmodule ExceptionTest do
     end
   end
 
-  if System.otp_release() >= "25" do
-    describe "binary constructor error info" do
-      defp concat(a, b), do: a <> b
+  describe "binary constructor error info" do
+    defp concat(a, b), do: a <> b
 
-      test "on binary concatenation" do
-        assert message(123, &concat(&1, "bar")) ==
-                 "construction of binary failed: segment 1 of type 'binary': expected a binary but got: 123"
+    test "on binary concatenation" do
+      assert message(123, &concat(&1, "bar")) ==
+               "construction of binary failed: segment 1 of type 'binary': expected a binary but got: 123"
 
-        assert message(~D[0001-02-03], &concat(&1, "bar")) ==
-                 "construction of binary failed: segment 1 of type 'binary': expected a binary but got: ~D[0001-02-03]"
-      end
+      assert message(~D[0001-02-03], &concat(&1, "bar")) ==
+               "construction of binary failed: segment 1 of type 'binary': expected a binary but got: ~D[0001-02-03]"
     end
   end
 
