@@ -14,7 +14,7 @@ defmodule FileTest do
     # Following Erlang's underlying implementation
     #
     # Renaming files
-    # :ok               -> rename file to existing file default behaviour
+    # :ok               -> rename file to existing file default behavior
     # {:error, :eisdir} -> rename file to existing empty dir
     # {:error, :eisdir} -> rename file to existing non-empty dir
     # :ok               -> rename file to non-existing location
@@ -26,7 +26,7 @@ defmodule FileTest do
     # :ok                -> rename dir to non-existing leaf location
     # {:error, ??}       -> rename dir to non-existing parent location
     # :ok                -> rename dir to itself
-    # :ok                -> rename dir to existing empty dir default behaviour
+    # :ok                -> rename dir to existing empty dir default behavior
     # {:error, :eexist}  -> rename dir to existing empty dir
     # {:error, :einval}  -> rename parent dir to existing sub dir
     # {:error, :einval}  -> rename parent dir to non-existing sub dir
@@ -35,7 +35,7 @@ defmodule FileTest do
     # other tests
     # {:error, :enoent} -> rename unknown source
     # :ok               -> rename preserves mode
-    test "rename file to existing file default behaviour" do
+    test "rename file to existing file default behavior" do
       src = tmp_fixture_path("file.txt")
       dest = tmp_path("tmp.file")
 
@@ -128,7 +128,7 @@ defmodule FileTest do
       end
     end
 
-    test "rename! file to existing file default behaviour" do
+    test "rename! file to existing file default behavior" do
       src = tmp_fixture_path("file.txt")
       dest = tmp_path("tmp.file")
 
@@ -256,7 +256,7 @@ defmodule FileTest do
       end
     end
 
-    test "rename dir to existing empty dir default behaviour" do
+    test "rename dir to existing empty dir default behavior" do
       src = tmp_fixture_path("cp_r")
       dest = tmp_path("tmp")
 
