@@ -30,6 +30,17 @@ defmodule Kernel.QuoteTest do
     assert quote(line: true, do: bar(1, 2, 3)) == {:bar, [line: __ENV__.line], [1, 2, 3]}
   end
 
+  test "file line" do
+    assert quote(file: "foo", line: 3, do: bar(1, 2, 3)) ==
+             {:bar, [keep: {"foo", 3}], [1, 2, 3]}
+
+    assert quote(file: "foo", line: false, do: bar(1, 2, 3)) ==
+             {:bar, [keep: {"foo", 0}], [1, 2, 3]}
+
+    assert quote(file: "foo", line: true, do: bar(1, 2, 3)) ==
+             {:bar, [keep: {"foo", __ENV__.line - 1}], [1, 2, 3]}
+  end
+
   test "quote line var" do
     line = __ENV__.line
     assert quote(line: line, do: bar(1, 2, 3)) == {:bar, [line: line], [1, 2, 3]}
@@ -60,6 +71,14 @@ defmodule Kernel.QuoteTest do
     end
   end
 
+  test "quote context bind_quoted" do
+    assert {:__block__, _,
+            [{:=, [], [{:some_var, _, :fallback}, 321]}, {:some_var, _, :fallback}]} =
+             (quote bind_quoted: [some_var: 321], context: __ENV__.context || :fallback do
+                some_var
+              end)
+  end
+
   test "operator precedence" do
     assert {:+, _, [{:+, _, [1, _]}, 1]} = quote(do: 1 + Foo.l() + 1)
     assert {:+, _, [1, {_, _, [{:+, _, [1]}]}]} = quote(do: 1 + Foo.l(+1))
@@ -120,6 +139,12 @@ defmodule Kernel.QuoteTest do
     assert {:quote, _, [[do: {:unquote, _, _}]]} = quote(do: quote(do: unquote(x)))
   end
 
+  test "import inside nested quote" do
+    # Check that we can evaluate imports from quote inside quote
+    assert {{:to_string, meta, [123]}, _} = Code.eval_quoted(quote(do: quote(do: to_string(123))))
+    assert meta[:imports] == [{1, Kernel}]
+  end
+
   defmacrop nested_quote_in_macro do
     x = 1
 
@@ -203,15 +228,6 @@ defmodule Kernel.QuoteTest do
 
     map = %{foo: :default}
     assert %{map | unquote_splicing(foo: :bar)} == %{foo: :bar}
-
-    assert Code.eval_string("quote do: %{unquote_splicing foo: :bar}") ==
-             {{:%{}, [], [foo: :bar]}, []}
-
-    assert Code.eval_string("quote do: %{:baz => :bat, unquote_splicing foo: :bar}") ==
-             {{:%{}, [], [{:baz, :bat}, {:foo, :bar}]}, []}
-
-    assert Code.eval_string("quote do: %{foo bar | baz}") ==
-             {{:%{}, [], [{:foo, [], [{:|, [], [{:bar, [], Elixir}, {:baz, [], Elixir}]}]}]}, []}
   end
 
   test "when" do
