@@ -5,8 +5,8 @@ tokenize(String) ->
   tokenize(String, []).
 
 tokenize(String, Opts) ->
-  {ok, _Line, _Column, _Warnings, Result} = elixir_tokenizer:tokenize(String, 1, Opts),
-  Result.
+  {ok, _Line, _Column, _Warnings, Result, []} = elixir_tokenizer:tokenize(String, 1, Opts),
+  lists:reverse(Result).
 
 tokenize_error(String) ->
   {error, Error, _, _, _} = elixir_tokenizer:tokenize(String, 1, []),
@@ -109,10 +109,6 @@ identifier_test() ->
 module_macro_test() ->
   [{identifier, {1, 1, _}, '__MODULE__'}] = tokenize("__MODULE__").
 
-triple_dot_test() ->
-  [{identifier, {1, 1, _}, '...'}] = tokenize("..."),
-  [{'.', {1, 1, nil}}, {identifier, {1, 3, _}, '..'}] = tokenize(". ..").
-
 dot_test() ->
   [{identifier, {1, 1, _}, foo},
    {'.', {1, 4, nil}},
@@ -159,16 +155,14 @@ aliases_test() ->
 
 string_test() ->
   [{bin_string, {1, 1, nil}, [<<"foo">>]}] = tokenize("\"foo\""),
-  [{bin_string, {1, 1, nil}, [<<"f\"">>]}] = tokenize("\"f\\\"\""),
-  [{list_string, {1, 1, nil}, [<<"foo">>]}] = tokenize("'foo'").
+  [{bin_string, {1, 1, nil}, [<<"f\"">>]}] = tokenize("\"f\\\"\"").
 
 heredoc_test() ->
   [{bin_heredoc, {1, 1, nil}, 0, [<<"heredoc\n">>]}] = tokenize("\"\"\"\nheredoc\n\"\"\""),
   [{bin_heredoc, {1, 1, nil}, 1, [<<"heredoc\n">>]}, {';', {3, 5, 0}}] = tokenize("\"\"\"\n heredoc\n \"\"\";").
 
 empty_string_test() ->
-  [{bin_string, {1, 1, nil}, [<<>>]}] = tokenize("\"\""),
-  [{list_string, {1, 1, nil}, [<<>>]}] = tokenize("''").
+  [{bin_string, {1, 1, nil}, [<<>>]}] = tokenize("\"\"").
 
 concat_test() ->
   [{identifier, {1, 1, _}, x},
