@@ -276,7 +276,7 @@ defmodule ExUnit.Case do
   """
 
   @type env :: module() | Macro.Env.t()
-
+  @compile {:no_warn_undefined, [IEx.Pry]}
   @reserved [:module, :file, :line, :test, :async, :registered, :describe]
 
   @doc false
@@ -366,34 +366,52 @@ defmodule ExUnit.Case do
     end
 
     contents =
-      case contents do
+      case annotate_test(contents, __CALLER__) do
         [do: block] ->
           quote do
             unquote(block)
             :ok
           end
 
-        _ ->
+        contents ->
           quote do
             try(unquote(contents))
             :ok
           end
       end
 
-    var = Macro.escape(var)
-    contents = Macro.escape(contents, unquote: true)
     %{module: mod, file: file, line: line} = __CALLER__
 
-    quote bind_quoted: [
-            var: var,
-            contents: contents,
-            message: message,
-            mod: mod,
-            file: file,
-            line: line
-          ] do
-      name = ExUnit.Case.register_test(mod, file, line, :test, message, [])
-      def unquote(name)(unquote(var)), do: unquote(contents)
+    name =
+      quote do
+        name =
+          ExUnit.Case.register_test(
+            unquote(mod),
+            unquote(file),
+            unquote(line),
+            :test,
+            unquote(message),
+            []
+          )
+      end
+
+    def =
+      {:def, [],
+       [
+         {{:unquote, [], [quote(do: name)]}, [], [var]},
+         [do: contents]
+       ]}
+
+    {:__block__, [], [name, def]}
+  end
+
+  defp annotate_test(contents, caller) do
+    if Application.get_env(:ex_unit, :breakpoints, false) and Keyword.keyword?(contents) do
+      for {key, expr} <- contents do
+        {key, IEx.Pry.annotate_quoted(expr, true, caller)}
+      end
+    else
+      contents
     end
   end
 
@@ -560,7 +578,7 @@ defmodule ExUnit.Case do
   display. You can use `ExUnit.plural_rule/2` to set a custom
   pluralization.
   """
-  @doc since: "1.10.0"
+  @doc since: "1.11.0"
   def register_test(mod, file, line, test_type, name, tags) do
     unless Module.has_attribute?(mod, :ex_unit_tests) do
       raise "cannot define #{test_type}. Please make sure you have invoked " <>
@@ -629,8 +647,8 @@ defmodule ExUnit.Case do
   This function is deprecated in favor of `register_test/6` which performs
   better under tight loops by avoiding `__ENV__`.
   """
-  # TODO: Deprecate on Elixir v1.17
-  @doc deprecated: "Use register_test/6 instead"
+  # TODO: Remove me Elixir v2.0
+  @deprecated "Use register_test/6 instead"
   @doc since: "1.3.0"
   def register_test(%{module: mod, file: file, line: line}, test_type, name, tags) do
     register_test(mod, file, line, test_type, name, tags)
