@@ -24,7 +24,7 @@ defmodule Mix.Project do
 
   ## Configuration
 
-  In order to configure Mix, the module that `use`s `Mix.Project` should export
+  In order to configure Mix, the module that calls `use Mix.Project` should export
   a `project/0` function that returns a keyword list representing configuration
   for the project.
 
@@ -325,6 +325,11 @@ defmodule Mix.Project do
   a full recompilation whenever such configuration files change.
   For this reason, the mtime is cached to avoid file system lookups.
 
+  However, for effective used of this function, you must avoid
+  comparing source files with the `config_mtime` itself. Instead,
+  store the previous `config_mtime` and compare it with the new
+  `config_mtime` in order to detect if something is stale.
+
   Note: before Elixir v1.13.0, the `mix.exs` file was also included
   in the mtimes, but not anymore. You can compute its modification
   date by calling `project_file/0`.
@@ -935,5 +940,5 @@ defmodule Mix.Project do
 
   @private_config [:build_scm, :deps_app_path, :deps_build_path]
   defp get_project_config(nil), do: []
-  defp get_project_config(atom), do: atom.project |> Keyword.drop(@private_config)
+  defp get_project_config(atom), do: atom.project() |> Keyword.drop(@private_config)
 end
